/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : OS_TrustZone_s.c
Purpose : embOS sample program for ARMv8M TrustZone
*/

#include "fsl_device_registers.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
//
// Non-secure addresses
//
#define FLASH_START_NS   (0x00010000u)
#define FLASH_SIZE_NS    (0x00072000u)
#define FLASH_START_NSC  (0x1000FE00u)
#define FLASH_SIZE_NSC   (0x00000200u)
#define RAM_START_NS     (0x20008000u)
#define RAM_SIZE_NS      (0x0002B000u)
#define PERIPH_START_NS  (0x40000000u)
#define PERIPH_SIZE_NS   (0x00100000u)

/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
__attribute__((cmse_nonsecure_entry)) void IncrementCounter_s(void);

/*********************************************************************
*
*       Types
*
**********************************************************************
*/
typedef void ResetHandler_t(void) __attribute__((cmse_nonsecure_call));

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static unsigned int Counter_s;

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SAU_setup()
*
*  Function description
*    Initializes the SAU
*/
static void SAU_setup(void) {
  SAU->CTRL = 0u;  // Disable SAU
  //
  // Configure SAU region 0 - Non-secure Flash for code execution
  //
  SAU->RNR  = 0;
  SAU->RBAR = (FLASH_START_NS & SAU_RBAR_BADDR_Msk);
  SAU->RLAR = ((FLASH_START_NS + FLASH_SIZE_NS - 1u) & SAU_RLAR_LADDR_Msk) | ((0u << SAU_RLAR_NSC_Pos) & SAU_RLAR_NSC_Msk) | ((1u << SAU_RLAR_ENABLE_Pos) & SAU_RLAR_ENABLE_Msk);
  //
  // Configure SAU region 1 - Non-secure RAM for data
  //
  SAU->RNR  = 1;
  SAU->RBAR = (RAM_START_NS & SAU_RBAR_BADDR_Msk);
  SAU->RLAR = ((RAM_START_NS + RAM_SIZE_NS-1) & SAU_RLAR_LADDR_Msk) | ((0u << SAU_RLAR_NSC_Pos) & SAU_RLAR_NSC_Msk) | ((1u << SAU_RLAR_ENABLE_Pos) & SAU_RLAR_ENABLE_Msk);
  //
  // Configure SAU region 2 - Non-secure callable Flash for code veneer table
  //
  SAU->RNR  = 2;
  SAU->RBAR = (FLASH_START_NSC & SAU_RBAR_BADDR_Msk);
  SAU->RLAR = ((FLASH_START_NSC + FLASH_SIZE_NSC - 1u) & SAU_RLAR_LADDR_Msk) | ((1u << SAU_RLAR_NSC_Pos) & SAU_RLAR_NSC_Msk) | ((1u << SAU_RLAR_ENABLE_Pos) & SAU_RLAR_ENABLE_Msk);
  //
  // Configure SAU region 3 - Non-secure peripherals address space
  //
  SAU->RNR  = 3;
  SAU->RBAR = (PERIPH_START_NS & SAU_RBAR_BADDR_Msk);
  SAU->RLAR = ((PERIPH_START_NS + PERIPH_SIZE_NS - 1u) & SAU_RLAR_LADDR_Msk) | ((0u << SAU_RLAR_NSC_Pos) & SAU_RLAR_NSC_Msk) | ((1u << SAU_RLAR_ENABLE_Pos) & SAU_RLAR_ENABLE_Msk);

  __DSB();         // Force memory writes before continuing
  __ISB();         // Flush and refill pipeline with updated permissions
  SAU->CTRL = 1u;  // Enable SAU
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       IncrementCounter_s()
*/
__attribute__((cmse_nonsecure_entry)) void IncrementCounter_s(void) {
  Counter_s++;
}

/*********************************************************************
*
*       main()
*/
int main(void) {
  ResetHandler_t* ResetHandler;

  SAU_setup();                                                        // Configure SAU/IDAU
  SCB->SHCSR  |= (1u << 19);                                          // Enable secure fault for easier debugging
  SCB_NS->VTOR = FLASH_START_NS ;                                     // Setup non-secure vector table
  __TZ_set_MSP_NS(*((uint32_t*)FLASH_START_NS));                      // Setup non-secure main stack
  //    
  // Call non-secure reset handler
  //
  ResetHandler = (ResetHandler_t*)*(uint32_t*)(FLASH_START_NS + 4u);  // Get non-secure reset handler
  ResetHandler();                                                     // Call non-secure reset handler
  return 0;
}

/*************************** End of file ****************************/
