/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2025 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.20.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : Initializes and handles the hardware for embOS
*/

#ifndef   BSP_USE_RTC
  #define BSP_USE_RTC  (0)  // Configuration: Use either SysTick/CycleCount (0, default) or RTC (1) as time base for embOS-Ultra
#endif

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"
#include "cc13x2-cc26x2-cm4.h"

#if (BSP_USE_RTC != 0)
#include "aon_event.h"
#include "aon_rtc.h"
#include "sys_ctrl.h"
#include "setup_rom.h"
#endif

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#if (BSP_USE_RTC != 0)

#define RTC_RSHIFT          11                             // RTC has 1 << 32 Hz. To be used with embOS, we need to right shift the RTC value in order to have
                                                           // a useful range for timers, since BSP_OS_StartTimer() uses a 32 bit argument.
                                                           // Therefore, if using the default RTC value without right shift, the maximum timer length of
                                                           // 0xFFFFFFFF cycles would equal to only one* second.
                                                           // *0.99999999976716935634613037109375
#define RTC_INC             0x2147B                        // default is 0x20000, which results in RTC being 4% behind real time
                                                           // adjust to used hardware if it skews differently

/*********************************************************************
*
*       Defines, not configurable
*
**********************************************************************
*/
#define RTC_INC_VAL         RTC_INC << 6                   // RTC_INC is stored as 2^38/freq, instead of 2^32/freq. Therefore, RTC_INC must be left shifted by 6.
#define RTC_HZ              (1 << 16) >> RTC_RSHIFT << 16
#define RTC_COMPARERSHIFT   16                             // Compare Value format is "a 16 bit integer and 16 bit fractional format <16 sec.16 subsec"
#if (RTC_RSHIFT > RTC_COMPARERSHIFT)
  #pragma error "Not allowed."
#endif

#endif  // BSP_USE_RTC

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_IF_JLINK
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKMEM.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  const OS_U32 OS_JLINKMEM_BufferSize = 32u;  // Size of the communication buffer for JLINKMEM
#else
  const OS_U32 OS_JLINKMEM_BufferSize = 0u;   // Buffer not used
#endif

uint32_t SystemCoreClock = 48000000u;

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       _OS_OnRX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static void _OS_OnRX(unsigned int Unit, unsigned char c) {
  OS_USE_PARA(Unit);
  OS_COM_OnRx(c);
}

/*********************************************************************
*
*       _OS_OnTX()
*
*  Function description
*    Callback wrapper function for BSP UART module.
*/
static int _OS_OnTX(unsigned int Unit) {
  OS_USE_PARA(Unit);
  return (int)OS_COM_OnTx();
}
#endif

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_OS_GetCycles()
*
*  Function description
*    Calculates the current time in counter cycles since reset.
*
*  Return value
*    current time in counter cycles since reset.
*/
OS_U64 BSP_OS_GetCycles(void) {
#if (BSP_USE_RTC != 0)
  return AONRTCCurrent64BitValueGet() >> RTC_RSHIFT;
#else
  OS_U32 TimerCycles;
  OS_U32 CycleCntLow;
  OS_U32 CycleCntHigh;
  OS_U32 CycleCntCompare;
  OS_U64 CycleCnt64;

  do {
    CycleCnt64   = OS_TIME_GetTimestamp();      // Read current OS_Global.Time
    CycleCntLow  = (OS_U32)CycleCnt64;          // Extract lower word of OS_Global.Time
    CycleCntHigh = (OS_U32)(CycleCnt64 >> 32);  // Extract higher word of OS_Global.Time
    TimerCycles  = DWT->CYCCNT;                 // Read current hardware CycleCount
    //
    // CycleCnt64 and TimerCycles need to be retrieved "simultaneously" for the below calculation to work:
    // If the above code is interrupted after retrieving OS_Global.Time, but before reading the hardware counter,
    // it may happen that the hardware counter overflows multiple times before returning here.
    // We could then no longer accurately compare (TimerCycles < CycleCntLow) below.
    //
    // Hence, we check if the upper word of OS_Global.Time still matches the value we retrieved earlier and repeat
    // the process if it doesn't. This works because OS_Global.Time is updated once per timer interrupt, and the timer
    // interrupt must be more frequent than the hardware counter overflow, so any overflow has necessarily incremented the upper word.
    //
    CycleCntCompare = (OS_U32)(OS_TIME_GetTimestamp() >> 32);
  } while (CycleCntHigh != CycleCntCompare);
  //
  // Calculate the current time in counter cycles since reset.
  // This is done by simply copying the 32 bit hardware time stamp into the lower 32 bits of OS_Global.Time, while
  // the upper 32 bits of OS_Global.Time count the number of overflows of the hardware counter.
  // Therefore, we compare the lower 32 bits of OS_Global.Time to the newly retrieved hardware time stamp:
  //
  if (TimerCycles < CycleCntLow) {
    //
    // The hardware counter has overflown and we must increment the upper 32 bits of OS_Global.Time.
    //
    CycleCnt64 = ((OS_U64)(CycleCntHigh + 1u) << 32) + TimerCycles;
  } else {
    //
    // The hardware counter has not overflown and we do not perform any additional action.
    //
    CycleCnt64 = ((OS_U64)CycleCntHigh << 32) + TimerCycles;
  }
  return CycleCnt64;
#endif
}

/*********************************************************************
*
*       BSP_OS_StartTimer()
*
*  Function description
*    Program the hardware timer required for embOS's time base.
*    The used hardware timer must generate an interrupt before the used counter completes a full loop.
*
*  Parameters
*    Cycles: The amount of cycles after which the hardware timer shall generate an interrupt.
*/
void BSP_OS_StartTimer(OS_U32 Cycles) {
#if (BSP_USE_RTC == 0)
  SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk
                | SysTick_CTRL_TICKINT_Msk;  // Disable SysTick
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  //
  // If embOSView via J-Link is selected, JLINKMEM_Process() needs to be executed periodically by the application.
  // In this exemplary BSP implementation, JLINKMEM_Process() is called from the SysTick_Handler().
  // We therefore limit the system tick to occur after a maximum of 10 milliseconds,
  // guaranteeing JLINKMEM_Process() is executed at least every 10 milliseconds.
  //
  if (Cycles > (SystemCoreClock / 100u)) {
    Cycles = SystemCoreClock / 100u;
  }
#endif
  SysTick->LOAD = Cycles;                    // Set reload register
  SysTick->VAL  = 0u;                        // Load the SysTick Counter Value
  SysTick->CTRL = SysTick_CTRL_ENABLE_Msk
                | SysTick_CTRL_CLKSOURCE_Msk
                | SysTick_CTRL_TICKINT_Msk;  // Enable SysTick
#else
  AONRTCChannelDisable(AON_RTC_CH0);
  AONRTCEventClear(AON_RTC_CH0);
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  //
  // If embOSView via J-Link is selected, JLINKMEM_Process() needs to be executed periodically by the application.
  // In this exemplary BSP implementation, JLINKMEM_Process() is called from the SysTick_Handler().
  // We therefore limit the system tick to occur after a maximum of 10 milliseconds,
  // guaranteeing JLINKMEM_Process() is executed at least every 10 milliseconds.
  //
  if (Cycles > (RTC_HZ / 100u)) {
    Cycles = RTC_HZ / 100u;
  }
#endif
  // Cycles = Cycles << RTC_RSHIFT;
  // Cycles = Cycles >> RTC_COMPARERSHIFT;
  Cycles = Cycles >> (RTC_COMPARERSHIFT - RTC_RSHIFT);
  AONRTCCompareValueSet(AON_RTC_CH0, AONRTCCurrentCompareValueGet() + Cycles);
  AONRTCChannelEnable(AON_RTC_CH0);
#endif
}

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
void SysTick_Handler(void) {
#if (BSP_USE_RTC == 0)
  SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk
                | SysTick_CTRL_TICKINT_Msk;  // Disable SysTick
  OS_INT_EnterNestable();
  OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKMEM_Process();
#endif
  OS_INT_LeaveNestable();
#else
   AONRTCChannelDisable(AON_RTC_CH0);
   AONRTCEventClear(AON_RTC_CH0);
   OS_INT_EnterNestable();
   OS_TICK_Handle();
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
   JLINKMEM_Process();
#endif
   OS_INT_LeaveNestable();
#endif
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Initialize NVIC vector table offset register if applicable for this device.
  // Might be necessary for RAM targets or application not running from 0x00.
  //
#if (defined(__VTOR_PRESENT) && (__VTOR_PRESENT == 1))
  extern int __Vectors;
  SCB->VTOR = (OS_U32)&__Vectors;
#endif
  //
  // Enable instruction and data cache if applicable for this device
  //
#if (defined(__ICACHE_PRESENT) && (__ICACHE_PRESENT == 1))
  SCB_EnableICache();
#endif
#if (defined(__DCACHE_PRESENT) && (__DCACHE_PRESENT == 1))
  SCB_EnableDCache();
#endif
  //
  // Inform embOS about the frequency of the counter
  //
  {
#if (BSP_USE_RTC == 0)
    OS_SYSTIMER_CONFIG SysTimerConfig = { SystemCoreClock };
#else
    OS_SYSTIMER_CONFIG SysTimerConfig = { RTC_HZ };
#endif
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Start the counter (here: cycle counter)
  //
#if (BSP_USE_RTC == 0)
  if ((CoreDebug->DEMCR & CoreDebug_DEMCR_TRCENA_Msk) == 0) {  // Trace not enabled?
    CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;            // Enable trace
  }
  if ((DWT->CTRL & DWT_CTRL_NOCYCCNT_Msk) == 0) {              // Cycle counter supported?
    if ((DWT->CTRL & DWT_CTRL_CYCCNTENA_Msk) == 0) {           // Cycle counter not enabled?
      DWT->CTRL |= DWT_CTRL_CYCCNTENA_Msk;                     // Enable Cycle counter
    }
  }
#else
  SysCtrlAonSync();
  AONRTCDisable();
  AONRTCEventClear(AON_RTC_CH0);
  AONRTCReset();
  AONEventMcuWakeUpSet(AON_EVENT_MCU_WU0, AON_EVENT_RTC_CH0);
  SetupSetAonRtcSubSecInc(RTC_INC_VAL);                         // Adjust RTC clock skew
  //
  // Use RTC channel 0 for embOS SysTick
  //
  AONRTCCombinedEventConfig(AON_RTC_CH0);
  IntRegister(INT_AON_RTC_COMB, SysTick_Handler);
  IntEnable(INT_AON_RTC_COMB);
  AONRTCEnable();
#endif
  //
  // Start the hardware timer (here: SysTick)
  //
#if (BSP_USE_RTC == 0)
  NVIC_SetPriority(SysTick_IRQn, (1u << __NVIC_PRIO_BITS) - 2u);  // Set the priority higher than the PendSV priority
#else
  NVIC_SetPriority(CC13XX_CC26XX_CM4_EXCEPTION_AON_RTC, (1u << __NVIC_PRIO_BITS) - 2u);  // Set the priority higher than the PendSV priority
#endif
  BSP_OS_StartTimer(0xFFFFFFFFu);
  //
  // Configure and initialize SEGGER SystemView
  //
#if (OS_SUPPORT_TRACE_API != 0)
  SEGGER_SYSVIEW_Conf();
#endif
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKMEM_SetpfOnRx(OS_COM_OnRx);
  JLINKMEM_SetpfOnTx(OS_COM_OnTx);
  JLINKMEM_SetpfGetNextChar(OS_COM_GetNextChar);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Init(OS_UART, OS_BAUDRATE, BSP_UART_DATA_BITS_8, BSP_UART_PARITY_NONE, BSP_UART_STOP_BITS_1);
  BSP_UART_SetReadCallback(OS_UART, _OS_OnRX);
  BSP_UART_SetWriteCallback(OS_UART, _OS_OnTX);
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*    It may be used to e.g. enter a low power mode of the device.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*    The idle loop can be exited only when an embOS interrupt causes
*    a context switch (e.g. after expiration of a task timeout),
*    hence embOS interrupts must not permanently be disabled.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
    #if ((OS_VIEW_IFSELECT != OS_VIEW_IF_JLINK) && (OS_DEBUG == 0))
      //
      // When uncommenting this line, please be aware device
      // specific issues could occur.
      // Therefore, we do not call __WFI() by default.
      //
      //__WFI();        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKMEM_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  BSP_UART_Write1(OS_UART, c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*************************** End of file ****************************/
